# -*- coding: utf-8 -*-
"""
Created on Tue Jul  4 10:09:47 2017

@author: akibdpt
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from builtins import range
from builtins import super
from builtins import str
from builtins import object

"""
transformation for parametric shape grammar

graphTransform consists of vector transformation sets which are sets of 
fromVects and toVects
when applying a graphTransform on a shape, any vector in points/lines/planes in
the shape that is equals to a fromVect in graphTransform will be transformed into
the coresponding toVect.
e.g. transform:    fromVect  |   toVect
                     (0,0)  -->  (3,3)
                     (0,1)  -->  (5,5)
                     (1,0)  -->  (6,0)
                     (1,1)  -->  (8,2)
    applying on shape: <(0,0),(0,1)>,<(0,0),(1,0)>,<(0,1),(1,1)>,<(1,0),(1,1)>
    resulting shape:   <(3,3),(5,5)>,<(3,3),(6,0)>,<(5,5),(8,2)>,<(6,0),(8,2)>
"""
class graphTransform(object):
    
    type = 'transform-graphTransform'
    
    """
    graphTransform consists of a list of vector transformations, 
    each vector transformation is a list with two elements, 
    1st element is fromVect and 2nd element is toVect
    
    e.g. self.vects = [[fromVect1, toVect1],
                       [fromVect2, toVect2],
                       [fromVect3, toVect3],
                       ...]
    """
    def __init__(self):
        self.vects = []
        self.unresolved = []
    
    def __repr__(self):
        return str(self)
    
    def __str__(self):
        s = '{'
        for i in self.vects:
            s+= str(i[0])+'->'+str(i[1])+','
        for i in self.unresolved:
            s+= str(i)+'-> ?,'
        s = s[:-1] + '}'
        return s
        
    def __len__(self):
        return len(self.vects)
            
            
    """
    add a set of vector transformation (fromVect and its corresponding toVect)
    **override: False: do not override the vector transformation if fromVect 
                       already exists in the current graphTransform
                '>' : override the vector transformation only if the new toVect 
                      is bigger than the current toVect corresponding with fromVect
                      in the current graphTransform
                '<' : override the vector transformation only if the new toVect 
                      is smaller than the current toVect corresponding with fromVect
                      in the current graphTransform
                True: override the vector transformation if fromVect already
                      exists in the current graphTransform
    """
    def add(self, fromVect, toVect, override = False):
        for i in self.vects:
            if i[0] == fromVect:
                if not i[1] == toVect:
                    if override == '>' and toVect > i[1]:
                        i[1] = toVect
                        return True
                    if override == '<' and toVect < i[1]:
                        i[1] = toVect
                        return True
                    elif override is True:
                        i[1] = toVect
                        return True
                    else:
                        return False
                return True
        self.vects.append([fromVect, toVect])
        return True
        
    def addUnresolved(self, fromVect):
        if fromVect not in self.unresolved:
            self.unresolved.append(fromVect)
        
    def duplicate(self):
        new = graphTransform()
        for i in self.vects:
            new.vects.append(i)
        for i in self.unresolved:
            new.unresolved.append(i)
        return new
        
    """
    print out the transform
    """
    def printTransf(self):
        for i in self.vects:
            print('  ',i[0],'->',i[1])
        for i in self.unresolved:
            print('  ',i,'-> ?')
            
    """
    apply the transform on a vector
    """
    def apply(self, vect):
        for i in self.vects:
            if i[0] == vect:
                return i[1]
        return vect
        
    """
    apply the transform on a vector
    """
    def transform(self, vect):
        for i in self.vects:
            if i[0] == vect:
                return i[1]
        return False
        
    """
    check if a vector has an existing transformation in this graphTransform
    """
    def contains(self, vect):
        for i in self.vects:
            if i[0] == vect:
                return True
        return False     
        
    """
    check if a vector is already a transformation for a from vector
    """
    def containsMatch(self, vect):
        for i in self.vects:
            if i[1] == vect:
                return True
        return False        
        
    """
    check if a vector has an existing transformation in this graphTransform
    """
    def hasCheckPointOf(self, line):
        for i in self.vects:
            if line.containsPoint(i[0]):
                return True
        return False
        
    def getCheckPoints(self, line):
        ints = []
        for i in self.vects:
            if line.containsPoint(i[0]):
                check = False
                for j in range(len(ints)):
                    if i[0] < ints[j][0]:
                        ints.insert(j, i)
                        check = True
                        break
                if check is False:
                    ints.append(i)
        return ints
        
    """
    find the fromVect that would be transformed into this vector
    """
    def findOrigin(self, vect):
        for i in self.vects:
            if i[1] == vect:
                return i[0]
        return False
        
    def transformable(self, ind, residuals):
        if 'ind' in ind.type:
            if 'ind-point' in ind.type:
                for i in self.vects:
                    if i[0] == ind.value:
                        return True
                residuals.append(ind)
                return False
            elif 'ind-lineSegment' in ind.type:
                cS = cE = False
                for i in self.vects:
                    if i[0] == ind.start:
                        cS = True
                    if i[0] == ind.end:
                        cE = True
                if cS and cE:
                    return True
                residuals.append(ind)
                return False
            elif 'ind-planeSegment' in ind.type:
                for b in ind.boundaries:
                    for p in b.polygonal:
                        check = False
                        for i in self.vects:
                            if i[0] == p:
                                check = True
                                break
                        if not check:
                            residuals.append(ind)
                            return False
            return True
        else:
            check = True
            if ind.type in ('form-metaForm','form-interval','form-areal','form-coForm'):
                for i in ind.elements:
                    if not self.transformable(i, residuals):
                        check = False
            elif ind.type == 'form-discrete' and 'ind-point' in ind.elements[0].type:
                for i in ind.elements:
                    if not self.transformable(i, residuals):
                        check = False
            return check
        