
# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

# Rhino Scripting
import rhinoscriptsyntax as rs

# Python built-ins
from builtins import range

# Sortal Library Imports
from sortal import sort, rule

# Import for transformations
from mpmath import pi, sin, cos
from copy import deepcopy

# Rhino-specific imports
from rhino_specific.draw_result import draw_result

# Conversion Classes Import
from convert.convert_rhino_to_agnostic import convertR
from convert.extract import extract

# API imports
import sortalgi as sgi

# Conversion Classes Initialization
convertR = convertR()       # Converts Rhino GUIDs to agnostic tuple form
extract = extract()         # Extracts Rhino GUIDs of objects inside a frame 

"""
===========
SCRIPT INFO
===========

This script is run after 'create_rule.py', and applies a chosen rule onto 
the initial shape/a subshape contained in a larger shape in the Rhino workspace. 
When running this code, click the green 'Play' button. Do not reset the engine as it will empty the sort register.

Rules are obtained from 'rule.register'; this register is stored in the sortal library.

======================
MAJOR SECTIONS OF CODE
======================

'apply_rule.py' has the following major sections:
    
1. Subshape and shape selection: select shape and subshape, convert these to agnostic forms
2. Rule selection: select rule to apply onto shape from rule register
3. Finding rule applications: generate match-result-application combinations based on subshape 
4. Application selection: select application to apply onto shape
5. Clearing previous shape and plotting new one: Clear previous shape and draw new shape after rule application
"""

"""
===============================
1. SHAPE AND SUBSHAPE SELECTION
===============================
This section prompts for the subshape and main shape (the subshape is checked to see if it is part of the main shape).
"""

# SUBSHAPE SELECTION: selecting individual Rhino objects that form subshape
# Extract Rhino objects of subshape; this will serve as the basis for match and result generation
subshape = rs.GetObjects('Select subshape as basis for matches')  

# Retrieve frame insertion point (needed for obtaining relative location to frame instance          
frame_instance = rs.GetObject('Select frame of subshape')                   
refPointSub = rs.BlockInstanceInsertPoint(frame_instance)  

# Convert Rhino GUIDs to agnostic form; input is list of GUIDs                 
subshapeAgnostic, subshapeIds = convertR.toAgnostic(subshape, refPointSub)
  
# SHAPE SELECTION: selecting main shape by selecting frame that main shape is contained in
# Extracts Rhino objects of main form
shape, refPointMain, layerNameMain  = extract.getIndividuals('Select the frame containing the main shape')
   
# Converts list of Rhino GUIDs to agnostic form and sorted GUIDs
shapeAgnostic, shapeIds = convertR.toAgnostic(shape, refPointMain)                                           

"""
=================
2. RULE SELECTION
=================
This section prompts for the selection of the rule to be applied. 
"""

# RETRIEVING AVAILABLE RULES 
# Prints which rules are already stored in rule.register 
print ('Rules available:')
count = 1
ruleRegisterList = []

for i in rule.register:
    print ('%i. '%count, i)
    ruleRegisterList.append(i) 
    count+=1
    
# Select rule to apply onto forms
choice = raw_input('Select rule by entering number of rule')

# Check if choice entered is a digit 
if choice.isdigit():
    choice = int(choice)
    
    while True:
        if choice > 0 and choice <= len(rule.register):
            chosenRule = ruleRegisterList[choice - 1]
            chosenRule = sgi.getRule(chosenRule)
            break
        else: 
            choice = raw_input('Incorrect input. Please enter choice again')
    
    """
    ============================
    3. FINDING RULE APPLICATIONS
    ============================
    The specific function for finding rule applications is 'find_rule_appns'
    If input is correct, then proceeds with finding rule applications
    
    'find_rule_appns' applies rule using match defined by subshape onto main shape
    
    Input: rule instance, main shape in agnostic form, subshape in agnostic form
    Output: tuple with the following format
    (<LHS match in agnostic form>, <RHS result in agnostic form>, <Main Shape after rule application in agnostic form>)
    
    The code for this function can be found in api -> agnostic_api -> find_rule_appns.py
    """
    ruleAppns = sgi.find_rule_appns(chosenRule, shapeAgnostic, subshapeAgnostic)

"""         
========================
4. APPLICATION SELECTION
========================

This section prints the applications available for performing onto the initial shape, 
and prompts for which application is to be performed on the user.

For the purposes of this demo, the results are printed in their agnostic tuple form.
"""
print ('\nAvailable rule applications', len(ruleAppns))
for appn in ruleAppns:
    print ('Application No. %i'%(ruleAppns.index(appn)+1))
    print ('\nMatrix: ', appn[0])
    print ('\nMatch : ', appn[1])
    print ('\nResult : ', appn[1])
    print ('\nApplication:', appn[2], '\n')
    
choice = raw_input('Select rule application by entering number')

while True:
    if choice.isdigit() and len(ruleAppns) != 0:
        choice = int(choice)
        if choice > 0 and choice <= len(ruleAppns):
            choice = choice - 1
            break
        else:
            choice = raw_input('Please enter only numbers. Please enter choice again')
            
    elif len(ruleAppns) == 0:
        print ('There are no rule applications found. Ending run')
        break
        
    else:
        choice = raw_input('Incorrect input. Please enter choice again')

"""
=================================================
5. CLEARING PREVIOUS SHAPE AND PLOTTING NEW SHAPE
=================================================
draw_result clears the initial shape (through the list of shapeIds),
and draws the result of the rule application as performed on the main shape.
This function can be found in api -> rhino_specific -> draw_result.

INPUT: list of shape IDs of shape to be redrawn, index of selected application in list 'ruleAppns',
layer name of where shape is draw, reference point of frame block instance where shape is to be drawn inside of

OUTPUT: list of Rhino GUIDs corresponding to redrawn shape
"""

shapeIdsNew = draw_result(shapeIds, choice, ruleAppns, layerNameMain, refPointMain)

