# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

# Rhino Scripting
import rhinoscriptsyntax as rs

# Python built-ins
from builtins import range

# Sortal Library Imports
from sortal import (transformSet, rule, primitiveSort, point3D, point2D, coordinate, lineSegment3D, planeSegment2D, 
                    lineSegment2D, line3D, label, rWeight, disjunctiveSort, attributeSort, affineTransform,  
                    vector3D, vector2D, planeSegment3D, sort)
                   
#from .setup.sortTypes import (pointSort3D, lineSegSort3D, descSort, weightSort, weight255Sort, labelSort, colorSort)
from sortalgi.sortal_lib_api.setup.sortTypes import (pointSort3D, lineSegSort3D, labelSort, weightSort, weight255Sort, colorSort, descSort, enumSort)

# Import for transformations
from mpmath import mp

"""
Convert CLASS
Sorts objects into points, line segments, with their accompanying attributes
Performs conversion of Rhino GUIDs to agnostic form

INPUT
Two options:
1 Free selection of Rhino objects (sample code below after the class definition)
2 From Layers + Frames (using extract Class from 'extract.py', inside convert file)

OUTPUT
Agnostic form dictionary and corresponding list of Rhino GUIDs, sorted into geometric object types 
(i.e. ({'points': [<points>], 'line segments'[<lines>]))

CURRENTLY SUPPORTS
Individuals: Points, lines and polylines
Attributes: Labels, descriptions, enumerative, weights (0 to 1, line widths), weights (grayscale from 0 to 255), color (RGB)
"""

class convertR(object):
    
    # Input: object GUIDs of form, insertion point of frame selected, disjunctive sort tuple
    # Object IDs - Rhino GUIDs; disjunctive sort tuple - what individuals and attributes (if any) are to compose the form
    # Returns agnostic form (dictionary) with format: {'points': [<p1>, <p2>], 'line segments': [<ls1>, <ls2>]}
    
    def __init__(self):
        pass
    
    # Default refPoint is origin
    def toAgnostic(self, objectIds, refPoint= (0,0,0)):
        self.objectIds = objectIds 
        self.lines = []                 # Contains GUIDs of all curve objects
        self.points = []                # Contains GUIDs of all point and text dot objects
        self.objects = []               # Contains GUIDs of sorted curve and point/text dot objects in following order: [[points], [line segments]]
        self.form = None                # Contains final agnostic form
        self.framePosition = refPoint
        self.discSort = sort.register['rhino_shapes']       # Sort type of disjunctive sort describing complete metaform/individual types in form
        self.mainSorts =  self.discSort.component           # List of sort types of main sorts
        
        # Checks if convert class received any objects
        if self.objectIds == None:
            ('No objects selected')
            return None        
           
        # Separates objects into individual types (points, lines, etc.) and handles selectively
        # based on metaform sort type
        self.separateObjects(self.objectIds, self.mainSorts)
        
        # Extracts data from object ids and sorts them according to individuals or a list of individuals (e.g. polylines)
        self.agnosticObject, self.formIds = self.transformObjects(self.objects)
        
        return self.agnosticObject, self.formIds

    """
    =================================
    CONVERTING GUIDs to AGNOSTIC FORM 
    =================================
    """
    
    # Separates objects into geometric ids and chooses which ones to handle, based on user sort type input
    # Example: if a user only wants to create a form of lines, this function will select 
    # only the lines/curve objects to put inside the form
    def separateObjects(self, objectIds, mainSorts):
        # Sorts objects into curves (line segments) and points/text dots
        # If block instance or polyline, explodes block instance and adds the GUIDs to the objectIds list
        for object in objectIds:
            if rs.IsCurve(object):
                self.lines.append(object)
                
            elif rs.IsPoint(object) or rs.IsTextDot(object):
                self.points.append(object)
                
            elif rs.IsBlockInstance(object):                
                name = rs.BlockInstanceName(object)
                blockObjects = rs.ExplodeBlockInstance(object)
                index = objectIds.index(object)
                objectIds.pop(index)
                
                for obj in blockObjects:
                    objectIds.append(obj)

        # Selective handling based on sort type specified by user
        check = False

        for sortName in mainSorts:
            if sortName.baseSort() == pointSort3D:
                self.objects.insert(0, self.points)
                
            if sortName.baseSort() == lineSegSort3D:
                self.objects.append(self.lines) 
                ind = self.objects.index(self.lines)
                check = True
    
        # Checks through object IDs to look for line segment - text dot pairs
        # Deletes text dots that are 'attached' to line segments from first sub-list of self.objects (points and text dots)
        # This is done to prevent the text dots that serve as labels for line segments, from being treated as labelled points
        if check:
            for line in self.lines:
                if rs.IsUserText(line) == 1: 
                    if 'LabelDot' in rs.GetUserText(line):
                        rs.DeleteObject(rs.GetUserText(line, 'LabelDot'))  
            
    # Transforms objects into agnostic form
    def transformObjects (self, objects):
        form = {}       # Stores agnostic individuals according to hash keys based on geometry type
        formIds = {}    # Stores corresponding GUIDs of agnostic individuals into corresponding lists based on geometry type
        
        # Sets up contents of dictionaries
        for indivSort in self.mainSorts:
            if 'lineSeg' in indivSort.name:
                form['line segments'] = []
                formIds['line segments'] = []
            elif 'point' in indivSort.name:
                form['points'] = []
                formIds['points'] = []

        for object in objects:
            for indiv in object:
                newObject = None    # Stores agnostic translation tuple of individual geometry
                objType = ''        # Internal key for processing individual geometry and its corresponding attributes
                
                # Checks type of GUID
                # If not point/line/polyline, then skipped over
                
                # Checks if GUID is line/polyline
                if rs.IsCurve(indiv):
                    
                    rgb = rs.coercecolor(rs.ObjectColor(indiv)) # Extracts color data from line segment
                    color = [rgb.R, rgb.G, rgb.B]
                    
                    # Handles individual line
                    # If line start == line end, then this creates an error and the code proceeds to handle the curve
                    # as a polyline
                    try:
                        indivType = 'lineSeg'
                        
                        # Determines attributes included
                        if rs.IsUserText(indiv) or rs.ObjectPrintWidth(indiv) >= 0.0 or color != [0,0,0]:
                            if 'Label' in rs.GetUserText(indiv):
                                indivType += 'Label'
                                
                            if 'Desc' in rs.GetUserText(indiv):
                                indivType += 'Desc'
                                
                            if 'Enum' in rs.GetUserText(indiv):
                                indivType += 'Enum'
                                
                            if rs.ObjectPrintWidth(indiv) >= 0.0:
                                indivType += 'Weight'

                            if color != [0,0,0]:
                                indivType += 'Color'

                        # Obtains line information (coords, attribute information) from line GUID
                        # Returns agnostic tuple form of line 
                        newObject = self.Line(indiv, indivType)

                    # Handles polyline
                    except:
                        indivType = 'polyline'
                        
                        # Determines attributes included
                        if rs.IsUserText(indiv) or rs.ObjectPrintWidth(indiv) >= 0.0 or color != [0,0,0]:
                            if 'Label' in rs.GetUserText(indiv):
                                indivType += 'Label'
                                
                            if 'Desc' in rs.GetUserText(indiv):
                                indivType += 'Desc'
                                
                            if 'Enum' in rs.GetUserText(indiv):
                                indivType += 'Enum'
                                
                            if rs.ObjectPrintWidth(indiv) >= 0.0:
                                indivType += 'Weight'

                            if color != [0,0,0]:
                                indivType += 'Color'
                                
                        # Obtains line information (coords, attribute information) from line GUID
                        # Returns agnostic tuple form of polyline 
                        newObject = self.Polyline(indiv, indivType)
                
                # Checks if GUID is point or text dot
                elif rs.IsPoint(indiv) or rs.IsTextDot(indiv):
                    color = rs.coercecolor(rs.ObjectColor(indiv))   # Extracts color data from point 
                    rgb = [color.R, color.G, color.B]
                    
                    # Assigns type of point
                    indivType = 'pt'
                    
                    # Determines attributes included
                    if rs.IsTextDot(indiv):
                        indivType += 'Label'
                        
                    if 'Desc' in rs.GetUserText(indiv):
                        indivType += 'Desc'
                        
                    if 'Enum' in rs.GetUserText(indiv):
                        indivType += 'Enum'

                    ## How to interpret weight vs. color
                    if  rgb != [0,0,0] and (rgb[0] == rgb[1] or rgb[1] == rgb[2] or rgb[0] == rgb[2]):
                        indivType += 'Weight'
                        
                    if  rgb != [0,0,0] and (rgb[0] != rgb[1] or rgb[1] != rgb[2] or rgb[0] != rgb[2]):
                        indivType += 'Color'

                    # Obtains point information (coords, attributes) from point GUID
                    # Returns agnostic tuple form of point
                    newObject = self.Point(indiv, indivType)
     
                # If newObject has been created, it is appended to form and its GUID is appended to formIds
                if newObject != None:
                    if 'lineSeg' in indivType or 'polyline' in indivType:
                        formIds['line segments'].append(indiv)      
                        form['line segments'].append(newObject)
                        
                    elif 'pt' in indivType:
                        formIds['points'].append(indiv)
                        form['points'].append(newObject)

        return form, formIds
    
    """
    ==========================
    EXTRACTING DATA from GUIDs
    ==========================
    """
    # Change: attributes tuple becomes attributes set
    # Extracts data from curve (line segment) GUID
    def Line(self, id, indivType):
        tail, head = 0, 0

        # Adjusts coordinates based on frame's Block Instance Insertion Point 
        p1_abs = rs.CurveStartPoint(id)
        p2_abs = rs.CurveEndPoint(id)
        p1_rel = rs.PointSubtract(p1_abs, self.framePosition)
        p2_rel = rs.PointSubtract(p2_abs, self.framePosition)
        
        # Determines order of plotting
        if p1_rel < p2_rel:
            tail = p1_rel
            head = p2_rel
        elif p1_rel > p2_rel:
            tail = p2_rel
            head = p1_rel
        else:
            raise ValueError
        
        # Creates vectors 
        v1 = vector3D(head[0], head[1], head[2])
        v2 = vector3D(tail[0], tail[1], tail[2])
        coords = ((v1,)(v2,))
        
        # Extracts attribute data
        # Stores each attribute type under corresponding hash key
        # Text attribute groups (labels, descriptions, enums) are stored as lists
        att = {}

        if 'Label' in indivType:
            text = rs.GetUserText(id, "Label")
            att['labels'] = [text]
            
        if 'Desc' in indivType:
            text = rs.GetUserText(id, "Desc")
            att['descriptions'] = [text]
            
        if 'Enum' in indivType:
            text = rs.GetUserText(id, "Enum")
            att['enums'] = [text]
            
        if 'Weight' in indivType:
            if rs.ObjectPrintWidth(id) == 0:
                weight = 0.01
            else:
                weight = rs.ObjectPrintWidth(id)
                weight = weight
            att ['weight'] = weight
            
        if 'Color' in indivType:
            color = rs.coercecolor(rs.ObjectColor(id))   # Obtains color of point or text dot 
            att['color'] = [int(color.R), int(color.G), int(color.B)]
            
        return (coords, att)
    
    # Extracts data from curve (polyline) GUID
    def Polyline(self, id, indivType):
        vertices = rs.PolylineVertices(id)
        vectors = ()

        # Adjusts coordinates based on frame's Block Instance Insertion Point 
        # Creates vectors
        for vector in vertices:
            vector_rel  = rs.PointSubtract(vector, self.framePosition)
            vectors += ((vector_rel.X, vector_rel.Y, vector_rel.Z),)

        # Extracts attribute data
        # Stores each attribute type under corresponding hash key
        # Text attributes groups (labels, descriptions, enums) are stored as lists
        att = {}
        
        if 'Label' in indivType:
            text = rs.GetUserText(id, "Label")
            att['labels'] = [text]
            
        if 'Desc' in indivType:
            text = rs.GetUserText(id, "Desc")
            att['descriptions'] = [text]
            
        if 'Enum' in indivType:
            text = rs.GetUserText(id, "Enum")
            att['enums'] = [text]
            
        if 'Weight' in indivType:
            if rs.ObjectPrintWidth(id) == 0:
                weight = 0.01
            else:
                weight = rs.ObjectPrintWidth(id)
                weight = weight
            att ['weight'] = weight
            
        if 'Color' in indivType:
            color = rs.coercecolor(rs.ObjectColor(id))   # Obtains color of point or text dot 
            att['color'] = [int(color.R), int(color.G), int(color.B)]

        return (vectors, att)
   
    # Extracts data from point (point/text dot) GUID
    def Point(self, id, indivType):
        # Obtains color information from point
        color = rs.coercecolor(rs.ObjectColor(id))
        rgb = [color.R, color.G, color.B]
        if rs.IsTextDot(id):
            coord_abs = rs.TextDotPoint(id) 
        else:
            coord_abs = rs.PointCoordinates(id)
    
        coord_rel = rs.PointSubtract(coord_abs, self.framePosition)
        coord = (coord_rel.X, coord_rel.Y, coord_rel.Z)
            
        # Extracts attribute data
        # Stores each attribute type under corresponding hash key
        # Text attributes (labels, descriptions, enums) are stored as lists
        att = {}

        if 'Label' in indivType:
            att['labels'] = [rs.TextDotText(id)]
       
        if 'Desc' in indivType:
            index = 'Desc'
            text = rs.GetUserText(id, index)
            att['descriptions'] = [text]
            
        if 'Enum' in indivType:
            index = 'Enum'
            text = rs.GetUserText(id, index)    
            att['enum'] = [text]
            
        # Weight - black (0,0,0) to white (255, 255, 255)
        if 'Weight' in indivType:           
            weight = rs.coercecolor(rs.ObjectColor(id))
            rgb = [int(weight.R), int(weight.G), int(weight.B)]
            if rgb[0] == rgb[1] and rgb[1] == rgb[2] and rgb[0] == rgb[2]:
                att['weight255'] = rgb
                
        # Color - normal rgb 0 to 255   
        if 'Color' in indivType:
            color = rs.coercecolor(rs.ObjectColor(id))  
            rgb = [int(color.R), int(color.G), int(color.B)]
            if rgb[0] != rgb[1] or rgb[1] != rgb[2] or rgb[0] != rgb[2]:
                att['color'] = rgb
                
        return (coord, att)