# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

import timeit

# Rhino Scripting
import rhinoscriptsyntax as rs

# Python built-in for range
from builtins import range
import copy

# Sortal Library Imports
from sortal import (transformSet, rule, primitiveSort, point3D, point2D, coordinate, lineSegment3D, planeSegment2D, 
                    lineSegment2D, line3D, label, rWeight, disjunctiveSort, attributeSort, affineTransform,  
                    vector3D, vector2D, planeSegment3D)
                    

# Import for transformations
from mpmath import pi, sin, cos

# Interpreter Imports
import setup.layer as l
import setup.settings as s
import setup.frame as f
import setup.arrow as a
import setup.grammar as g


"""
EXTRACT CLASS
Takes frame block instance as input, and extracts object GUIDs from frame block instances
Returns a list of Rhino GUIDs inside the frame

NOTES:
Code has been extracted and adapted from grammar interpreter code by Andrew Li's team
Supports line segments, polylines, text dots, points
Change layer of objects to intended layer first before running the code
"""

class extract:
    
    # Main function calling other functions inside extract class
    # Checks if each frame has any objects
    def getIndividuals (self, text = ''):
        frame_instance = rs.GetObject(text)
        if frame_instance != None:
            if rs.ObjectType(frame_instance) != 4096:
                rs.MessageBox ('Please select only frame block objects')
          
                return None
       
            else:
                # Extracts object GUIDs per frame
                objectIds = self._extract_elements(frame_instance)
                refPoint = rs.BlockInstanceInsertPoint(frame_instance)     
                layerName = rs.ObjectLayer(frame_instance)
                return objectIds, refPoint, layerName
        else:
            rs.MessageBox ('Please select only frame block objects')
            return None
    
    # Extracts objects inside frame instance selected
    def _extract_elements(
        self, frame_instance
    ):
        """
        Receives:
            frame_instance
                guid. A block instance (a cube) that contains the object GUIDs 
                making up the shape.
                            
        Returns:
            extractedObjects
                [guid, ...]. A list of GUIDs inside the selected frame block instance.
        """
        ill_formed_components = []

        elements = self._get_elements(frame_instance)
        if elements == []:
            ill_formed_components.append(frame_instance)
        else:
            extractedObjects = elements

        if ill_formed_components:
            error_message = "%s %s" % (
                "The following initial shapes and rules were omitted", 
                "because they contained inappropriate empty shapes")
            ill_formed_components_string = ',  '.join(str(ill_formed_components))
            total_message = "%s:\n\n%s" % (
                error_message, ill_formed_components_string)
            rs.MessageBox(total_message)
            
        return extractedObjects

 
    # Extracts GUIDs of elements inside layer
    def _get_elements(self, frame_instance):
        """
        Receives:
            frame_instance  str. The name of a frame instance
        Returns:
            elements        [guid, ...]. A list of the guids of the elements 
                            in the frame instance. 
        """
        objects_on_layer = l.Layer.get_objects_on_layer(frame_instance)
        elements = self._extract_elements_in_frame(
            frame_instance, objects_on_layer)
            
        return elements
        
    # Extracts GUIDs of elements inside frame, after setting layer in self._get_elements
    def _extract_elements_in_frame(self, frame_instance, objects_on_layer):
        """
        Receives:
            frame_instance  
                            guid. The guid of a frame instance
            objects_on_layer
                            [guid, ...]. A list of the guids of the objects  
                            on the layer containing the frame instance
        Returns:
            elements_in_frame
                            [guid, ...]. A list of the line, points and textdot guids 
                            in the frame instance
        """
        elements_in_frame = []
        frame_position = f.Frame.get_instance_position(frame_instance)
        frame_size = s.Settings.frame_size
        for object_guid in objects_on_layer:
            if (self._is_element(object_guid) and
                self._object_is_in_box(
                    object_guid, frame_position, frame_size)
            ):
                element_guid = object_guid
                elements_in_frame.append(element_guid)
                
        return elements_in_frame

    # Checks if the object is a curve (line/polyline) or a point(point/textdot)
    def _is_element(self, object_guid):
        """
        Receives:
            object_guid     guid. The guid of an object
        Returns:
            value           boolean. True, if the object is a curve (line/polyline) or a point
                            (point/textdot). False, otherwise
        """
        curve_type = 4
        textdot_type = 8192
        point_type = 1
        value = (
            rs.ObjectType(object_guid) == curve_type or 
            rs.ObjectType(object_guid) == textdot_type or
            rs.ObjectType(object_guid) == point_type)
            
        return value

    
    # Checks if object is inside a particular frame instance (based on box_position, box_size)
    def _object_is_in_box(self, object_guid, box_position, box_size):
        """
        Receives:
            object_guid     guid. The guid of an object
            box_position    Point3d. The position of the frame instance
            box_size        (num, num, num). The size of the frame block
        """
        curve_type = 4
        textdot_type = 8192
        point_type = 1
        if rs.ObjectType(object_guid) == curve_type:
            tail = rs.CurveStartPoint(object_guid)
            head = rs.CurveEndPoint(object_guid)
            points = [tail, head]
        elif rs.ObjectType(object_guid) == textdot_type:
            point = rs.TextDotPoint(object_guid)
            points = [point]
        elif rs.ObjectType(object_guid) == point_type: 
            point = rs.PointCoordinates(object_guid)
            points = [point]
        else:
            pass
        value = True
        for point in points:
            if not self._point_is_in_box(point, box_position, box_size):
                value = False
                break
                
        return value
        
    # Checks if point is inside a particular frame instance (based on position, size)
    def _point_is_in_box(self, point, position, size):
        """
        Receives:
            point           Point3d or (num, num, num)
            position        Point3d or (num, num, num). The position of the 
                            box
            size            Point3d or (num, num, num). The diagonal size of 
                            the box
        Returns:
            value           boolean. True, if the point is inside or on the 
                            surface of the box
        """
        p1 = position
        p2 = rs.PointAdd(p1, size)
        x0, y0, z0 = point
        x1, y1, z1 = p1
        x2, y2, z2 = p2
        value = (
            x1 <= x0 <= x2 and
            y1 <= y0 <= y2 and 
            z1 <= z0 <= z2)
            
        return value

"""
EXTRACTION TEST: Remove triple double quotes below if testing extraction -> conversion -> visualization 

Run the set_up_new_grammar file first, and then draw figures inside the frames.
Please draw objects inside the frames and change the objects to their corresponding layers first before running the code.
extract() prompts for selection of a frame instance with the shape inside.
It returns a list of GUIDs referring to the Rhino objects inside the frame, and the block insertion point of the frame block instance.
"""

#extract = extract()
#objectIds, refPoint = extract.getIndividuals()

