# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

import os

import timeit

# Rhino Scripting
import rhinoscriptsyntax as rs

# Python built-in imports
from builtins import range

# Sortal Library Imports
from sortal import (rule, sort, cleanup)
                    
# Import for transformations
from mpmath import pi, sin, cos

# Interpreter Imports 
from setup.grammar import Grammar as g
from setup.settings import Settings as s

"""
========================
RHINO SPECIFIC FUNCTIONS
========================

1. addLabel - adds label to Rhino Object's user text, visualized as text dot

2. addDesc - adds description to Rhino Object's user text, not visualized

3. addEnum - adds enumerative value to Rhino Object's user text, not visualized

4. delLabel - removes label from Rhino Object's user text; deletes text dot 'attached' to line segment, or reverts text dot to a point

5. attachLabel - attaches text dot to a particular line segment; this stores the text dot's GUID in the line segment's user text and moves the text dot

6. addWeight - adds width (weight) to line or grayscale (weight255) to point or text dot 

7. addColor - adds color to Rhino Object (one or several can be selected) using RGB values; this is not visible in Print Preview

8. delete - uses rhino's DeleteObjects to delete all Rhino objects in window, clears sort and rule registers, sets up new grammar frames

9. clearFrame - clears a frame block instance of all Rhino objects inside it; prompts for the user to select the frame

10. clearShape - clears a list of Rhino objects; takes as input a list/dictionary of Rhino GUIDs OR single Rhino GUID

"""

class rhino_methods: 

    @staticmethod
    # Adds label to user test of Rhino object
    def addLabel():
        id = rs.GetObject("Select object to place label")
        
        try:
            orig_color = rs.ObjectColor(id)
            rs.ObjectColor(id, [255,225,0])
            text_obj = raw_input('Enter label text for highlighted object')
            obj_type = '' 
            
            if rs.ObjectType(id) == 4 or rs.ObjectType(id) == 1 or rs.ObjectType(id) == 8192:
            
                # Handles adding labels to lines
                if rs.ObjectType(id) == 4:
                    obj_type = 'line segment'
                    
                    if text_obj != '':
                        if rs.IsUserText(id):
                            rs.DeleteObject(rs.GetUserText(id, "LabelDot"))
                        
                        rs.SetUserText(id, "Label", text_obj)    
                        textdot = rs.AddTextDot(text_obj, rs.CurveMidPoint(id))
                        rs.SetUserText(id, "LabelDot", textdot)
                        rs.ObjectColor(id, orig_color)
                        
                    else:
                        rs.SetUserText(id, "Label", '')
                        rs.DeleteObject(rs.GetUserText(id, "LabelDot"))
                            
                # Handles adding labels to simple points
                elif rs.ObjectType(id) == 1:
                    obj_type = 'point'
                    
                    if text_obj != '':
                        rs.SetUserText(id, "Label", text_obj)
                        rs.AddTextDot(text_obj, rs.PointCoordinates(id))
                        rs.DeleteObject(id)
                        rs.ObjectColor(id, orig_color)
                
                # Handles changing text of text dots
                elif rs.ObjectType(id) == 8192:
                    obj_type = 'text dot'
                    
                    if text_obj != '':
                        rs.SetUserText(id, "Label", text_obj)
                        rs.TextDotText(id, text_obj)
                        rgb = rs.coercecolor(rs.ObjectColor(id))
                        
                        if rgb != [0,0,0]:
                            color = rgb
                            rs.SetUserText(id, "Color", color)
                        rs.ObjectColor(id, orig_color)
                    else:
                        rs.AddPoint(rs.TextDotPoint(id))
                        rs.DeleteObject(id)
                
                string = text_obj + ' has been added to the ' + obj_type
                return string
        except:
            print ('No object selected')
    
    @staticmethod
    def addDesc():
        id = rs.GetObject('Select object to add description to')    
        
        if id != None:
            while True:
                print ('Is the description to be processed as a string or tuple?\nEnter 1 if tuple\nEnter 2 if string')
                choice = raw_input('Enter choice')
                
                if choice == '1':
                    while True:
                        desc = raw_input('Enter description enclosed by brackets ().')
                        if desc[0] == '(' or desc[-1] == ')':
                            desc = '`'+desc+'`'
                            rs.SetUserText(id, 'Desc', desc)
                            break
                        else:
                            print ('Wrong input. Please enter description enclosed by brackets ().')
                    break
                elif choice == '2':
                    desc = raw_input('Enter description as string')
                    desc = '`'+desc+'`' 
                    rs.SetUserText(id, 'Desc', desc)
                    break
                else:
                    print ('Wrong input. Please enter either only 1 or 2.')
                    
            print (desc + ' was entered as a description.')
        else:
            print ('No object selected.')
    
    @staticmethod
    def addEnum():
        id = rs.GetObject('Select object to add enumerative value to')    
        
        if id != None:
            print ('Enter single value to be entered as enumerative input. This will be read as a string.')
            enumText = raw_input('Enter enumerative value')
            rs.SetUserText(id, 'Enum', enumText)
            
            print (enumText + ' was entered as an enumerative value.')
        else:
            print ('No object selected.')
    
    @staticmethod
    # Deletes label from object and removes text dot
    def delLabel():
        id = rs.GetObject("Select object to remove label from")
        
        rs.SetUserText(id, 'Label')
        
        if rs.ObjectType(id) == 4:
            try:
                rs.DeleteObject(rs.GetUserText(id, "LabelDot"))
            except:
                pass
            
        elif rs.ObjectType(id) == 1:
            pass
            
        elif rs.ObjectType(id) == 8192:
            rs.AddPoint(rs.TextDotPoint(id))
            rs.DeleteObject(id)
    
    @staticmethod
    # Attaches a text dot label to a line
    def attachLabel():
        id = rs.GetObject('Select line segment to attach label to')
        label = rs.GetObject('Select label to attach to line segment')
        
        if rs.ObjectType(id) == 4 and rs.ObjectType(label) == 8192:
            ptText = rs.TextDotText(label)
            rs.DeleteObject(label)
            label = rs.AddTextDot(ptText, rs.CurveMidPoint(id))
            rs.SetUserText(id, 'LabelDot', label)   
            rs.SetUserText(id, 'Label', ptText)
            
        else: 
            print ("This method only applies to line segments and text dots.")
     
    @staticmethod
    # Adds width to curve object or grayscale to point
    def addWeight():
        inp = raw_input('Change weight of one object or several objects? Input ''object'' for one object or ''sObjects'' for several objects')
        
        # Handles several objects
        if inp == 'sObjects':
            ids =  rs.GetObjects("Select objects to change weight")
            lines, points = [], []
            rs.ObjectColor(ids, [255,225,0])                             # Highlights objects selected
            
            for id in ids:
                if rs.ObjectType(id) == 1 or rs.ObjectType(id) == 8192:     # Sorts objects into points or lines
                    points.append(id)
                elif rs.ObjectType(id) == 4:
                    lines.append(id)
            
            # Adds width to lines, if any
            if len(lines) !=0:   
                weight = raw_input('Enter weight for selected line/s (RANGE: 0.0 to 2.0mm)')
                try:
                    weight = float(weight)
                except:
                    print('Invalid input for weight.  Line widths are only decmal values from 0.0 to 2.0mm.')
                    return
                    
                if weight < 0.0 or weight > 2.0 or type(weight) != type(0.0):
                    print('Invalid input for weight.  Line widths are only decmal values from 0.0 to 2.0mm.')
                    return
                else:
                    for line in lines:
                        rs.ObjectPrintWidth(line, weight)
                    
            # Adds grayscale to points, if any    
            if len(points) !=0:
                rs.ObjectColor(points, [255,225,0])  
                weight= raw_input('Enter color for highlighted point/s (RANGE: 0 to 255)')
                try:
                    weight = int(weight)
                except:
                    print('Invalid input for weight.  Point weights are only integers from 0 (black) to 255 (white).')
                    return
                    
                if (rs.ObjectType(id) == 1 or rs.ObjectType(id) == 8192) and (weight < 0.0 or weight > 255 or type(weight) != type(0)):
                    print('Invalid input for weight.  Point weights are only integers from 0 (black) to 255 (white).')
                    return
                else:
                    # Changes color of point
                    rs.ObjectColor(id, (weight, weight, weight))
                    
        # Handles one object       
        elif inp == 'object':
            id = rs.GetObject("Select object to change weight")
            orig_color = rs.ObjectColor(id)
            rs.ObjectColor(id, [255,225,0])
            
            # Adds width to line, if any
            if rs.ObjectType(id) == 4:
                weight= raw_input('Enter weight for selected line/s (RANGE: 0.0 to 2.0mm)')
                try:
                    weight = float(weight)
                except:
                    print('Invalid input for weight.  Line widths are only decimals values from 0.0 to 2.0mm.')
                    return
                    
            # Adds grayscale to point/text dot, if any        
            elif (rs.ObjectType(id) == 1 or rs.ObjectType(id) == 8192):
                weight= raw_input('Enter color for selected point/s (RANGE: 0 to 255)')
                try:
                    weight = int(weight)
                except:
                    print('Invalid input for weight.  Point weights are only integers from 0 (black) to 255 (white).')
                    return
                    
            if rs.ObjectType(id) == 4 and (weight < 0.0 or weight > 2.0 or type(weight) != type(0.0)):
                print('Invalid input for weight.  Line widths are only decmal values from 0.0 to 2.0mm.')
                return
                
            elif (rs.ObjectType(id) == 1 or rs.ObjectType(id) == 8192) and (weight < 0.0 or weight > 255 or type(weight) != type(0)):
                print('Invalid input for weight.  Point weights are only integers from 0 (black) to 255 (white).')
                return
                
            else:    
                if rs.ObjectType(id) == 4:                                  # Changes line width
                    rs.ObjectPrintWidth(id, weight)
                
                elif rs.ObjectType(id) == 1 or rs.ObjectType(id) == 8192:   # Changes point/text dot color
                    rs.ObjectColor(id, (weight, weight, weight))
                    rs.SetUserText(id, "Color", str(weight), True)
        else:
            print('This method does not exist')
            return
    
    @staticmethod
    # Changes color of an object or several objects by selecting the object/s and inputting the RGB value
    # Note: Color is not visible in Print Preview
    def addColor():
        objectIds = rs.GetObjects('Select which objects to change color of')
        color = raw_input('Enter the RGB value for the color in the format <r>,<g>,<b> (no spaces and no parentheses)')
        color = color.split(',')
    
        if len(color)== 3 and color[0].isdigit() and color[1].isdigit() and color[2].isdigit():
            count = 0 
            for i in range(len(color)):
                color[i] = int(color[i])
                if color[i] <= 255 and color[i] >= 0:
                    
                    count+=1
            if count == 3:
                rs.ObjectColor(objectIds, (color[0], color[1], color[2]))
        else:
            print('Incorrect input for color.')
            
    @staticmethod
    # Deletes all Rhino objects 
    # Sets up new frames
    # Cleans up sort and rule registers
    def delete(): 
        rs.DeleteObjects(rs.AllObjects())
        g.set_up_grammar() 
        sort.cleanup()
        rule.cleanup()
     
    @staticmethod
    # Empties frame of Rhino objects
    def clearFrame():
        frameObjects, refPointL = extract.getIndividuals('Select frame to clear')
        
        for i in frameObjects:
            rs.DeleteObject(i)
        
        print ('<<< Frame cleared >>>')
        return
    
    @staticmethod
    # Removes the list of shape ids associated with a form
    # Input can be in a list, tuple, dictionary or single-item form
    def clearShape(shapeIds):
        try:
            for i in shapeIds:
                if type(i) == type([]):
                    for j in i:
                        rs.DeleteObjects(j)
                elif type(i) == type('a'):
                    #print ('\n', shapeIds[i])
                    rs.DeleteObjects(shapeIds[i])
                else:
                    rs.DeleteObject(i)
        except:
            rs.DeleteObject(shapeIds)
        
#addLabel()
#addWeight()
#addDesc()
#addEnum()
#attachLabel()
#attachLabel()
#addColor()
