# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

import rhinoscriptsyntax as rs
import Rhino
from builtins import range

from sortal import (transformSet, rule, primitiveSort, point3D, point2D, coordinate, lineSegment3D, lineSegment2D, 
                    line3D, label, rWeight, disjunctiveSort, attributeSort, affineTransform,  vector3D)

from mpmath import pi, sin, cos

"""
VisualA Class
Takes agnostic form as input and draws it in Rhino
Returns Rhino object GUIDs based on agnostic form

INPUT
Agnostic form dictionary, e.g. {'points': [<p1>, <p2>], 'line segments': [<ls1>, <ls2>]}

OUTPUT
Draws objects in Rhino
Returns list of GUIDs corresponding to Rhino objects just drawn
"""

class visualA:
    
    # Sets class instance variables to inputs and sets layer to be drawn on
    # Default value of refPoint is origin (0,0,0)
    def __init__ (self, agnosticObject, currentLayer, refPoint = (0,0,0)):
        self.agnosticObject = agnosticObject
        self.objectIds = []
        self.currentLayer = currentLayer
        rs.CurrentLayer(self.currentLayer)
        self.refPoint = rs.AddPoint(refPoint)
    
    # Draws agnostic form as Rhino objects
    def plotAgnostic(self):
        
        for i in self.agnosticObject:
            objects = self.agnosticObject[i]

            # Plots points/text dots
            if i == 'points':
                
                for obj in objects:
                    pt = rs.AddPoint(obj[0][0], obj[0][1], obj[0][2])
                    attributes = obj[1]
                    
                    for attrib in attributes:
                        content = attributes[attrib]

                        if attrib == 'labels':
                           
                            
                            totalText = ''
                            if len(attributes[attrib]) > 1:
                                for a in attributes[attrib]:
                                    if totalText == '':
                                        totalText = a
                                    else:
                                        totalText +=  (', '+ a)
                            else:
                                totalText = content[0] 
                            
                            #locPt =  Rhino.Geometry.Point3d(obj[0][0], obj[0][1], obj[0][2])
                            #pt = (rs.AddText(totalText, locPt, height=2.0, font="Arial", font_style=0, justification=None))
   
                            tempPt = rs.AddTextDot(totalText, (obj[0][0], obj[0][1], obj[0][2]))
                            color = rs.coercecolor(rs.ObjectColor(pt))
 
                            if color.R != 0 or color.G != 0 or color.R != 0:
                                rs.ObjectColor(tempPt, (color.R, color.G, color.B))
                                
                            if rs.IsUserText(pt) == 1: 
                                if 'Desc' in rs.GetUserText(indiv):
                                    text =  rs.GetUserText(pt, 'Desc')
                                    rs.SetUserText(tempPt, 'Desc', text)
                            
                                if 'Enum' in rs.GetUserText(indiv):
                                    text =  rs.GetUserText(pt, 'Enum')
                                    rs.SetUserText(tempPt, 'Enum', text)
                            
                            rs.DeleteObject(pt)
                            pt = tempPt

                        else: 
                            if attrib == 'weight255':
                                weight = content
                                rs.ObjectColor(pt, (float(weight[0]), float(weight[1]), float(weight[2])))
                                
                            elif attrib == 'descriptions':
                                rs.SetUserText(pt, 'Desc', content)
                            
                            elif attrib == 'enums':
                                rs.SetUserText(pt, 'Enum', content)
                                
                            elif attrib == 'color':
                                rs.ObjectColor(pt, (float(content[0]), float(content[1]), float(content[2])))
                                
                    self.objectIds.append(pt)
            
            # Plots line segments with/without attributes
            elif i == 'line segments':
                
                for obj in objects:
                    info = obj[0]
                    attributes = obj[1]
                    v1 = (obj[0][0][0], obj[0][0][1], obj[0][0][2])
                    v2 = (obj[0][1][0], obj[0][1][1], obj[0][1][2])
                    line = rs.AddLine(v1, v2)
                    
                    for attrib in attributes:
                        content = attributes[attrib]
                        
                        if attrib == 'labels':
                            midpt = rs.CurveMidPoint(line) 
                            totalText = ''
                            
                            if len(content) > 1:
                                for a in attributes[attrib]:
                                    if totalText == '':
                                        totalText = a
                                    else:
                                        totalText +=  (', '+ a)
                            else:
                                totalText = content[0] 
                                
                            textDot = rs.AddTextDot(totalText, midpt)
                            
                            self.objectIds.append(textDot)
                            rs.SetUserText(line, 'LabelDot', textDot)
                            rs.SetUserText(line, 'Label', content)

                        elif attrib == 'weight':   
                            width = float(content)
                            rs.ObjectPrintWidth(line, width)
                            
                        elif attrib == 'descriptions':
                            rs.SetUserText(line, 'Desc', content)
                        
                        elif attrib == 'enums':
                            rs.SetUserText(line, 'Enum', content)
                            
                        elif attrib == 'color':
                            rs.ObjectColor(line, (float(content[0]), float(content[1]), float(content[2])))
                                
                    self.objectIds.append(line)
        
        # Moves set of Rhino objects using refPoint as basis; 
        # e.g. can be used to redraw a form after rule application in the same location
        rs.MoveObjects(self.objectIds, self.refPoint)
        
        # Deletes refPoint from Rhino window
        rs.DeleteObject(self.refPoint)  
        
        return self.objectIds
    