# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

import timeit

# Rhino Scripting
import rhinoscriptsyntax as rs

# Import for transformations
from mpmath import pi, sin, cos

# Interpreter Imports
from . import settings as s
from . import frame as f
from . import arrow as a
from . import layer as l

"""
GRAMMAR CLASS
Sets up frames and layers for initial shape/s and rule sides

Code has been extracted and adapted from grammar interpreter code by Andrew Li's team
"""

class Grammar(object):
    
    def __init__(self):
        pass

    ### new
    @classmethod                                ##  done 08-08
    def set_up_grammar(cls):
        cls.clear_all()
        cls._set_up_first_initial_shape()
        cls._set_up_first_rule()
    
        ### utilities
    @classmethod                                ##  called
    def clear_all(cls):                         ##  system-created blocks only?
        cls._clear_objects()
        cls._clear_blocks()
        cls._clear_layers()
        cls._clear_data()
    
    @classmethod                                ##  called
    def _clear_objects(cls):
        """
        Deletes all drawn objects. 
        Returns:
            n_objects       int. The number of objects deleted, if successful
        """
        include_lights = True
        include_grips = True
        objects = rs.AllObjects(include_lights, include_grips)
        n_objects = rs.DeleteObjects(objects)
        return n_objects
    
    @classmethod                                ##  called
    def _clear_blocks(cls):
        block_names = rs.BlockNames()
        for name in block_names:
            rs.DeleteBlock(name)
    
    @classmethod                                ##  called
    def _clear_layers(cls):                     ##  clear only user layers
        """
        Deletes all layers. Leaves Default layer
        """
        default_layer_name = s.Settings.default_layer_name
        layer_names = rs.LayerNames()
        if not default_layer_name in layer_names:
            rs.AddLayer(default_layer_name)
        rs.CurrentLayer(default_layer_name)
        for layer_name in layer_names:
            if not layer_name == default_layer_name:
                rs.DeleteLayer(layer_name)
    
    @classmethod                                ##  called
    def _clear_data(cls):
        rs.DeleteDocumentData()
    
    @classmethod                                ##  done 08-06
    def _set_up_first_initial_shape(cls):
        """
        Required state:
            There must be no layer with the first initial shape layer name
            Adds a new layer with the first initial shape layer name and one frame 
            instance. Should be executed only once. 
        Returns:
            layer_name      str. The name of the layer, if successful. None, 
                            otherwise
        """
        method_name = '_set_up_first_initial_shape'
        try:
            layer_name = s.Settings.first_initial_shape_layer_name
            (   layer_name_is_in_use
            ) = (
                rs.IsLayer(layer_name))
            if layer_name_is_in_use:
                raise ValueError
        except ValueError:
            message = "The layer name '%s' is in use" % layer_name

            return_value = None
        else:
            frame_instance_position = (
                s.Settings.first_initial_shape_frame_position)
            return_value = cls._set_up_initial_shape(
                layer_name, frame_instance_position)
        finally:
            return return_value
    
    @classmethod                                ##  done 08-06
    def _set_up_initial_shape(cls, layer_name, frame_position):
        """
        Receives:
            layer_name      str. A well-formed and available layer name
            frame_position  Point3d. The position of the frame instance
            
        Creates a new layer with the specified name. Inserts a frame instance 
        on the new layer at the specified position. 
        
        Returns:
            layer_name_out  str. The name of the layer
        """
        l.Layer.new(layer_name)
        frame_instance_out = f.Frame.new_instance(layer_name, frame_position)
        layer_name_out = rs.ObjectLayer(frame_instance_out)
        return layer_name_out
    
    @classmethod
    def _set_up_first_rule(cls):                ##  done 08-06
        """
        Required state:
            There must be no layer with the first rule layer name
            Adds a new layer with the first rule layer name and two frame 
            instances. Should be executed only once. 
        
        Returns:
            layer_name      str. The name of the layer, if successful. None, 
                            otherwise
        """
        method_name = '_set_up_first_rule'
        try:
            layer_name = s.Settings.first_rule_layer_name
            (   layer_name_is_in_use
            ) = (
                rs.IsLayer(layer_name))
            if layer_name_is_in_use:
                raise ValueError
        except ValueError:
            message = "The layer name '%s' is in use" % layer_name

            return_value = None
        else:
            (   left_frame_position
            ) = (
                s.Settings.first_rule_left_frame_position)
            return_value = cls._set_up_rule(layer_name, left_frame_position)

        return return_value
        
        
    @classmethod                                ##  done 08-06
    def _set_up_rule(cls, layer_name, left_frame_position):
        """
        Receives:
            layer_name              str. A well-formed and available layer name
            left_frame_position     Point3d. The position of the left frame instance
        
        Creates a new layer with the specified name. Inserts left frame, right
        frame, and arrow instances on the new layer. 
        Returns:
            layer_name_out  str. The name of the rule layer
        """
        l.Layer.new(layer_name)
        (   right_frame_position,
            arrow_position
        ) = (   
            s.Settings.get_right_frame_position(left_frame_position),
            s.Settings.get_arrow_position(left_frame_position))
        f.Frame.new_instance(layer_name, left_frame_position)
        f.Frame.new_instance(layer_name, right_frame_position)
        arrow_out = a.Arrow.new_instance(layer_name, arrow_position)
        layer_name_out = rs.ObjectLayer(arrow_out)
        return layer_name_out