    #-*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

# Rhino Scripting
import rhinoscriptsyntax as rs
import System

# Python built-ins
from builtins import range

# Sortal Library Imports
from sortal import (transformSet, rule, primitiveSort, point3D, point2D, coordinate, lineSegment3D, planeSegment2D, 
                    lineSegment2D, line3D, label, rWeight, disjunctiveSort, attributeSort, affineTransform,  
                    vector3D, vector2D, planeSegment3D, sort, affineTransform3D)
            
# Import for transformations
from mpmath import pi, sin, cos
from copy import deepcopy

# API imports
from .sortal_lib_api.setup.parse_disjunctive_sort import parse_disjunctive_sort
from .sortal_lib_api.convert.convert_sortal_to_agnostic import convertS
from .sortal_lib_api.convert.convert_agnostic_to_sortal import convertA

convertA = convertA()
convertS = convertS()

"""
==============
ABOUT SORTALGI
==============

This module contains methods that allow the user to interact with the sortal library using rule names and 
agnostic shapes as input. 

1. sortal_setup() -  sets up sortal library background data; this must be run at the start whenever using the sortal library

2. create_rule() - creates a rule object from the inputs rule name, rule description, LHS agnostic object and RHS agnostic tuple

3. find_rule_appns() - takes as input rule object or rule name, subshape agnostic tuple and main shape agnostic tuple 
                       to generate rule applications

4. maximalize() - maximalizes an agnostic object

5. partOf() - checks of a subshape agnostic object is part of a potentially larger shape agnostic object

6. getRuleName() - returns name of rule instance

7. getRuleDesc() - returns description of rule instance

8. getLHS() - returns LHS of rule instance as an agnostic tuple

9. getRHS() - returns RHS of rule instance as an agnostic tuple

10. getRule() - retrieves rule object based on rule name input

11. setRuleName() - sets rule name of rule instance; if new rule name matches that of a 
                   pre-existing rule in the library, then the rule name is not changed

12. setRuleDesc() - sets description of rule instance

13. overwriteRule() - maintains rule name and rule description but changes one or both sides of the rule instance

"""

"""
=============
FUNCTION INFO
=============

'set_up_all.py' has the following functions:
    1. Setting up empty grammar frames for initial shape and rule; this can be commented out if one wishes 
       to use a pre-existing .3dm grammar file
    2. Initializing the sort types in the sortal library; this is necessary for rule creation and application

===================    
RUNNING THE SCRIPT:
===================
    To run this script, click on the small downward pointing arrow next to the green 'Play' button and select 'Reset engine and Debug'.
    This empties the sort register and rule register, and initializes new sort types.

=================
SORT TYPE SET UP:
=================
Creates disjunctive sort type  based on user input
This example enables:
      Index 0: Points with label, description, weight, weight255, enumerative, color
      Index 1: Line segments with label, description, weight, weight255, enumerative, color
      
However, note that only weight 255 (as grayscale) is visualized for points, and 
only weight (line width as viewed in Print Preview), are visualized in this demo 
"""

def sortal_setup(valueSet = ((True, True, True, True, True, True, True), (True, True, True, True, True, True, True))):
    pds = parse_disjunctive_sort() 
    pds.parseDisjSort(valueSet[0], valueSet[1])
    
    return True
    
# A simpler way of initializing sort types:
# discSort, mainSorts, attribSorts = pds.parseDisjSort((True, True), (True, False, True, False, True))
# This example enables:
#       Points with label
#       Line segments with description, weight, enumerative
# If the inputs for a geometric object's attributes are consecutive False values, they do not need to be inputted
# However, if there is a False value followed by a True value, it needs to be written explicitly, as in the line segment section
# So to enable a line segment with color as its only attribute, the tuple of True/False values needs to be: (True, False, False, False, False, False, True)

"""
=============
FUNCTION INFO
=============

create_rule() creates a new rule instance; it takes as input the rule name, rule description, and LHS and RHS agnostic shapes. 
Since create_rule() is linked to the sortal library, the act of creating a rule object checks if a rule with the same name
as the new rule being created already exists. If yes, then the rule is not created. 

INPUT
Rule name, rule description, lhs and rhs in agnostic form

OUTPUT
Rule instance and ruleName

Data Flow: agnostic tuple -> sortal metaform -> rule instance creation
"""

def create_rule(ruleName, ruleDesc, lhsAgnostic, rhsAgnostic):
    
    # Converts LHS and RHS agnostic tuples to sortal metaforms
    lhsMetaform = convertA.toSortal(lhsAgnostic)
    rhsMetaform = convertA.toSortal(rhsAgnostic)
    
    # Creates rule instance using sortal library
    ruleNew = rule(ruleName, ruleDesc, lhsMetaform, rhsMetaform)  
    
    if ruleNew.isValid():
        print (ruleName + ' has been created.')
        return ruleNew, ruleName
    else:
        message = ('Rule was not created since there was insufficient information for LHS Shape.\nThe minimum requirements for defining the LHS are:\n'+
                                '- 3 points (not coincident)\n'+
                                '- 1 point + 1 line (not co-linear)\n'+
                                '- 3 lines (no two parallel: all lines intersect one another)\n'+
                                '- 3 lines (2 lines parallel with each other, the other line intersecting both)\n'+
                                '- 2 lines (skew): However, this may have unexpected results; try not to have rules with only skew lines if possible')
                                
        System.Windows.Forms.MessageBox.Show(message, 'Invalid Rule Definition')
        del rule.register[ruleName]
        return None, None
 
"""
=============
FUNCTION INFO
=============
'find_rule_appns' finds rule applications from subshape or from a set of GUIDs 

INPUT
Rule instance or rule name (string), main shape agnostic tuple, subshape agnostic tuple (optional)

OUTPUT
List of applications (in tuple form) as based on match generated by subshapeAgnostic (agnostic version of subshape input) or shapeAgnostic, if no subshape input is given
Each element of the list is a tuple in the following format:
(<transformation matrix>, <LHS match in agnostic form based on subshape>, <RHS result in agnostic form>, <Main Shape after rule application in agnostic form>)
    
Data Flow: GUIDs -> agnostic -> sortal -> rule applications based on subshape, subshapeAgnostic
"""

def find_rule_appns(chosenRule, shapeAgnostic, subshapeAgnostic = None):
    try:
        if chosenRule.type == rule.type:
            pass
            
    except:
        if type(chosenRule) == type('a'):
            if chosenRule in rule.register:
                chosenRule = rule.register[chosenRule]
            else:
                raise ValueError('Rule name is not present in the rule register.')
        else:
            raise ValueError('Rule input is neither a rule object or a rule name.')
    
    # Conversion from agnostic to sortal form
    shapeSortal = convertA.toSortal(shapeAgnostic) 
    ruleAppns = []
    
    if subshapeAgnostic != None:
        subshapeSortal = convertA.toSortal(subshapeAgnostic)
    
        # Checks if subshape is part of main shape before proceeding
        if subshapeSortal.partOf(shapeSortal):
            subshapeApps = chosenRule.detect(subshapeSortal)       # Generates matches, results and rule applications for subshape
            
            for index1 in range(len(subshapeApps)):
                matrix = subshapeApps[index1].transformSet.retrieveTransform(affineTransform3D).matrix
                match = convertS.toAgnostic(subshapeApps[index1].matchForm)
                result = convertS.toAgnostic(subshapeApps[index1].resultForm)
                after = convertS.toAgnostic(subshapeApps[index1].perform(shapeSortal.duplicate()))
                ruleAppns.append((matrix, match, result, after,))
        
        else:
            message = ('Subshape is not part of main shape. Matches cannot be found.')
            rs.MessageBox(message)
    else:
        
        shapeApps = chosenRule.detect(shapeSortal)       # Generates matches, results and rule applications for subshape
            
        for index1 in range(len(shapeApps)):
            matrix = shapeApps[index1].transformSet.retrieveTransform(affineTransform3D).matrix
            match = convertS.toAgnostic(shapeApps[index1].matchForm)
            result = convertS.toAgnostic(shapeApps[index1].resultForm)
            after = convertS.toAgnostic(shapeApps[index1].perform(shapeSortal.duplicate()))
            ruleAppns.append((matrix, match, result, after,))
        
    return ruleAppns

"""
=============
FUNCTION INFO
=============
'maximalize' returns a maximalized agnostic tuple 

INPUT
Agnostic tuple

OUTPUT
Maximalized tuple
    
Data Flow: agnostic -> sortal -> agnostic
"""

def maximalize (shapeAgnostic):
    try:
        shapeSortal = convertA.toSortal(shapeAgnostic)
        shapeSortal.maximalize()
        shapeAgnostic = convertS.toAgnostic(shapeSortal)
        
        return shapeAgnostic
    except:
        return None

"""
=============
FUNCTION INFO
=============
'partOf' checks if a subshape agnostic tuple is part of a possibly larger shape agnostic tuple

INPUT
Agnostic tuples

OUTPUT
Boolean value True/False
    
Data Flow: agnostic -> sortal -> Boolean Output
"""
   
def partOf(subshapeAgnostic, shapeAgnostic):
    shapeSortal = convertA.toSortal(shapeAgnostic)
    subshapeSortal = convertA.toSortal(subshapeAgnostic)
    
    return subshapeSortal.partOf(shapeSortal)

"""
=============
FUNCTION INFO
=============
The next four functions return a specific aspect of a rule object, as per the name of the method

INPUT
Rule object or rule name (getRuleDesc, getLHS, getRHS)

OUTPUT
Name, description, LHS or RHS (as agnostic tuple)
    
Data Flow: rule object -> string or agnostic tuple
"""

def getRuleName(ruleObj):
    try:
        return ruleObj.name
    except:
        message = ('Input must be a rule object.')
        rs.MessageBox(message) 
        return None
    
def getRuleDesc(ruleObj):
    try:
        return ruleObj.description
    except:
        if type(ruleObj) == type('a'):
            return rule.register[ruleObj].description
        else:
            message = ('Input must be either a rule name or rule object.')
            rs.MessageBox(message) 
            return None
            
def getLHS(ruleObj):
    try:
        lhs = convertS.toAgnostic(ruleObj.lhs)
        
        return lhs
    except:
        if type(ruleObj) == type('a'):
            return rule.register[ruleObj].lhs
        else:
            message = ('Input must be either a rule name or rule object.')
            rs.MessageBox(message) 
            return None

def getRHS(ruleObj):
    try:
        rhs = convertS.toAgnostic(ruleObj.rhs)
        
        return rhs
    except:
        if type(ruleObj) == type('a'):
            return rule.register[ruleObj].rhs
        else:
            message = ('Input must be either a rule name or rule object.')
            rs.MessageBox(message) 
            return None

"""
=============
FUNCTION INFO
=============
getRule retrieves the corresponding rule object based on the input rule name from the sortal library.  

INPUT
Rule name

OUTPUT
Rule object
    
Data Flow: rule name -> rule object
"""  

def getRule(ruleName):
    if type(ruleName) == type('a'):
        if ruleName in rule.register:
            return rule.register[ruleName]
        else:
            message = ('Rule does not exist.')
            rs.MessageBox(message)    
    else:
        message = ('Name entered is not a string.')
        rs.MessageBox(message)
        
    return None

"""
=============
FUNCTION INFO
=============
setRuleName changes the name of a rule object; however, if the new rule name matches that of a pre-existing rule, 
this will cause an error. If successful, the previous rule name is deleted from the sortal library's rule register
and is free to use for new rules.

INPUT
Rule object, name(string)

OUTPUT
Altered rule object with new rule name in sortal library's rule register
    
Data Flow: rule object -> altered rule object
"""
    
def setRuleName(ruleObj, name):
    if type(name) == type('a'):
        ruleObj.setName(name)
        return ruleObj
    else:
        message = ('Name entered is not a string.')
        rs.MessageBox(message)
        return None
        
"""
=============
FUNCTION INFO
=============
setRuleDesc changes the description of a rule object.

INPUT
Rule object, description(string)

OUTPUT
Altered rule object with new description
    
Data Flow: rule object -> altered rule object
"""     
   
def setRuleDesc(ruleObj, desc):
    if type(desc) == type('a'):
        ruleObj.setDescription(desc)
        return ruleObj
    else:
        message = ('Name entered is not a string.')
        rs.MessageBox(message)
        return None

"""
=============
FUNCTION INFO
=============
overwriteRule changes the LHS shape, RHS shape or both sides of a rule object.  
It keeps the rule object's original name and description, and returns a rule object 
with the altered LHS and RHS shapes.

INPUT
Rule object, LHS (agnostic object), RHS (agnostic object)

OUTPUT
Altered rule object with the same name and description as the original rule object,
but with a different LHS and RHS
    
Data Flow: rule object -> altered rule object
"""  
    
def overwriteRule(ruleObj, lhs, rhs):
    try:
        lhs = convertA.toSortal(lhs)
        rhs = convertA.toSortal(rhs)
        name = ruleObj.name
        desc = ruleObj.description
        
        del rule.register[name]
        
        ruleObj = rule(name, desc, lhs, rhs)
        
        return ruleObj
    except:
        message = ''
        try:
            test =  ruleObj.name
        except:
            message +=('Rule inputted is not a rule object.')

        try:
            lhs = convertA.toSortal(lhs)
        
        except:
            message += ('LHS inputted is not an agnostic object\n')
            
        try:
            rhs = convertA.toSortal(rhs)
        except:
            message += ('RHS inputted is not an agnostic object\n')
        
        rs.MessageBox(message)
        return None
            


    
