# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

# Rhino Scripting
import rhinoscriptsyntax as rs

# Python built-ins
from builtins import range

# Sortal Library Imports
from sortal import (transformSet, rule, primitiveSort, point3D, point2D, coordinate, lineSegment3D, planeSegment2D, 
                    lineSegment2D, line3D, label, rWeight, disjunctiveSort, attributeSort, affineTransform,  
                    vector3D, vector2D, planeSegment3D, description, color, label, enumerative, sort)
                    
# User-defined sort type imports from sort_types
from ..setup.sortTypes import (pointSort3D, lineSegSort3D, labelSort, weightSort, weight255Sort, colorSort, descSort, enumSort)
# add in user defined enum sort later

# Import for transformations
from mpmath import pi, sin, cos

# API Imports
from ..sortal_shortcuts.create import create

# API Initialization
create = create()

"""
ConvertA CLASS
Sorts objects into points, line segments, labels
Performs conversion of agnostic dictionary to sortal grammar

INPUT
Agnostic dictionary

OUTPUT
Sortal metaform

CURRENTLY SUPPORTS
Individuals: Points, lines and polylines
Attributes: Labels, descriptions, enumerative, weights (0 to 1, line widths), weights (grayscale from 0 to 255), color (RGB)
"""

class convertA(object):
    
    # Input: agnostic form, discursive sort tuple
    # Converts agnostic form to sortal metaform

    def __init__(self):
        pass

    def toSortal(self, agnosticObject):
        self.agnosticObject = agnosticObject                            # Contains agnostic form
        self.discSort = sort.register['rhino_shapes']                   # Sort type of disjunctive sort which contains entire metaform
        self.mainSorts =  self.discSort.component                       # List of sort types of main sorts retrieved from disjunctive sort
        
        # Checks if convert class received any objects
        if self.agnosticObject == None:
            ('No input')
            return None        
        
        # Transforms agnostic objects into sortal individuals
        individuals, revisedMainSorts = self.makeIndividuals(self.agnosticObject, self.discSort, self.mainSorts)
        
        # Creates metaform and inserts forms of individuals inside
        self.metaform = self.makeForms(individuals, self.discSort, revisedMainSorts)
        
        return self.metaform
    
    """
    ======================================
    CONVERTING GUIDs to SORTAL INDIVIDUALS 
    ======================================
    """

    # Creates sortal individuals from extracted data in agnostic object 
    def makeIndividuals(self, agnosticObject, discSort, mainSorts):
        individuals = [[],[]]
        revisedMainSorts = []

        for i in agnosticObject:
            for sortType in mainSorts:
                
                # Assigns mainSortIndex and appends it to revisedMainSorts 
                # based on the geometric object type inside the agnostic object
                if i == 'points':
                    if 'point' in sortType.name:
                        mainSortIndex = sortType
                        revisedMainSorts.append(sortType)
                    index = 0
                elif i == 'line segments':
                    if 'line' in sortType.name:
                        mainSortIndex = sortType
                        revisedMainSorts.append(sortType)
                    index = 1
                    
            # Creates point individuals with corresponding attributes
            if 'points' in i:
                for obj in agnosticObject['points']:
                    coords = obj[0]
                    attributes = obj[1]
                    individual = point3D(mainSortIndex, coords[0], coords[1], coords[2]) # individual w/o attributes
                    att = [] 
                    
                    # Checks if attributes are present
                    if len (attributes) > 0:
                        for attrib in attributes:
                            
                            # labels
                            if attrib == 'labels' and 'label' in mainSortIndex.name:
                                if len(attributes[attrib]) > 0: 
                                    for content in attributes[attrib]:
                                        att.append(label(labelSort, content))
                            
                            # descriptions
                            elif attrib == 'descriptions' and 'desc' in mainSortIndex.name:
                                if len(attributes[attrib]) > 0: 
                                    for content in attributes[attrib]:
                                        att.append(description(descSort, content))
                            
                            # enums
                            elif attrib == 'enums' and 'enum' in mainSortIndex.name:
                                if len(attributes[attrib]) > 0: 
                                    for content in attributes[attrib]:
                                        att.append(enumerative(enumSort, content))
                            
                            # weight - based on Rhino weight
                            elif attrib == 'weight' and 'weight' in mainSortIndex.name: 
                                att.append(rWeight(weightSort, attributes[attrib]))
                            
                            # weight255 - white to black (grayscale)
                            elif attrib == 'weight255' and 'weight255' in mainSortIndex.name:    
                                att.append(rWeight(weight255Sort, attributes[attrib][0]))
                            
                            # color - rgb values
                            elif attrib == 'color' and 'color' in mainSortIndex.name:
                                rgb = attributes[attrib]  
                                att.append(color(colorSort, int(rgb[0]), int(rgb[1]), int(rgb[2])))
    
                        if len(att) != 0:
                            for a in att:
                                individual.addAttribute(a)
                                
                    individuals[index].append(individual)
                    
            # Creates line segments individuals with corresponding attributes
            elif 'line segments' in i:

                for obj in agnosticObject['line segments']:
                    
                    if type(obj[0][0]) == type (()):
                        info = obj[0]
                        attributes = obj[1]

                        v1 = vector3D(obj[0][0][0], obj[0][0][1], obj[0][0][2])
                        v2 = vector3D(obj[0][1][0], obj[0][1][1], obj[0][1][2])
                        individual = lineSegment3D(mainSortIndex, v1, v2)
                        att = []
                    
                    # Checks if attributes are present
                    if len (attributes) > 0:
                        for attrib in attributes:
                            
                            # labels 
                            if attrib == 'labels' and 'label' in mainSortIndex.name:
                                if len(attributes[attrib]) > 0: 
                                    for content in attributes[attrib]:
                                        att.append(label(labelSort, content))
                            
                            # descriptions
                            elif attrib == 'descriptions' and 'desc' in mainSortIndex.name:
                                if len(attributes[attrib]) > 0: 
                                    for content in attributes[attrib]:
                                        att.append(description(descSort, content))
                            
                            # enums
                            elif attrib == 'enums' and 'enum' in mainSortIndex.name:
                                if len(attributes[attrib]) > 0: 
                                    for content in attributes[attrib]:
                                        att.append(enumerative(enumSort, content))
                            
                            # weight - line width
                            elif attrib == 'weight' and 'weight' in mainSortIndex.name: 
                                att.append(rWeight(weightSort, attributes[attrib]))
                            
                            # weight 255 - white to black (grayscale)
                            elif attrib == 'weight255' and 'weight255' in mainSortIndex.name:    
                                att.append(rWeight(weight255Sort, attributes[attrib][0]))
                            
                            # color - rgb values
                            elif attrib == 'color' and 'color' in mainSortIndex.name:
                                rgb = attributes[attrib]  
                                att.append(color(colorSort, int(rgb[0]), int(rgb[1]), int(rgb[2])))
                        
                        # Inserts attributes into sortal individual if any
                        if len(att) != 0:
                            for a in att:
                                individual.addAttribute(a)

                    individuals[index].append(individual)
        
        # Returns list of sortal individiuals, main sort types arranged by geometric object type
        return individuals, revisedMainSorts      
    
    """
    =================
    CREATING METAFORM
    =================
    """
    
    # Creates forms to be inserted into metaform
    def makeForms(self, individuals, discSort, revisedMainSorts):
        forms = []

        for sortType in revisedMainSorts:                   # Assigns list of individuals to be inserted into form
            if 'point' in sortType.name:
                indiv = individuals [0]
            elif 'lineSeg' in sortType.name:
                indiv = individuals [1]
                
            form = sortType.newForm()                       # Creates one form per sublist
            
            for object in indiv:                            # Inserts individuals into form
                if type(object)==type(individuals):         # If there are several individuals, then treats it as a list
                    for obj in object:
                        form.insert(obj)
                else:                                       # If there is only one individual, then it is inserted 
                    form.insert(object)                     # into the form
                     
            forms.append(form)                              # Appends finished subform to list of forms that will make up
                                                            # the metaform

        formM1 = create.metaForm(discSort, forms)           # Creates metaform and inserts forms into metaform
        return formM1
   
    


