# -*- coding: utf-8 -*-
"""
@author: akidribm
"""

# RHINO SPECIFIC IMPORTS | FROM PYTHON 3.5 to 2.7
from __future__ import print_function
from __future__ import division
from __future__ import unicode_literals
from __future__ import absolute_import

# Rhino Scripting
import rhinoscriptsyntax as rs

# Python built-ins
from builtins import range

# Sortal Library Imports
from sortal import (transformSet, rule, primitiveSort, point3D, point2D, coordinate, lineSegment3D, planeSegment2D, 
                    lineSegment2D, line3D, label, rWeight, disjunctiveSort, attributeSort, affineTransform,  
                    vector3D, vector2D, planeSegment3D, enumerative, sort)
                    
# User-defined sort type imports from sort_types
from .sortTypes import (pointSort3D, lineSegSort3D, descSort, labelSort, weightSort, weight255Sort, colorSort, enumSort)

# The class 'parse_disjunctive_sort' currently supports initializing the following sort types:
# GEOMETRIC SUPPORT - point, line segment
# ATTRIBUTE SUPPORT (True/False values in this order) - label, description, weight, weight255, enumerative, color

class parse_disjunctive_sort:
    
    # Creates sort types for metaform and forms of individuals to be placed inside the metaform
    # Based on user input
    def parseDisjSort(self, point = (False, False, False, False, False, False, False), line = (False, False, False, False, False, False, False)):

        check = True
        mainSorts = []   # Completed sorts
        attribSorts = [] # Attribute sorts for corresponding index of completed sorts
        geomSorts = [point, line]
        
        for ind in range(len(geomSorts)):
            if geomSorts[ind][0]:
                fullSortName = ''      # Name of full sort 
                baseSort, fullSortName = self.mainSort(ind, fullSortName)        # Base sort which becomes fullSort if no attributes
                attributes = geomSorts[ind][1:]
                attribName, attribSort, fullAttribSort = self.attributeSorts(attributes, baseSort)
                fullSortName = fullSortName + '_' + attribName
                
                # base sort - base sort
                # fullsortname - base sort + attribute sort
                
                # attribName - name of attribute sort 
                # attribSort - attribute sort components
                # fullAttribSort - full disjunctive attribute sort
                
                # Checks if full attribute sort already exists
                # If yes, takes previously determined attribute sort and assigns it to fullSort
                # If no, creates attributeSort to assign to fullSort
                if attributeSort.checkSort(fullSortName) == False:
                    fullSort = attributeSort(fullSortName, baseSort, fullAttribSort)
                else:
                    fullSort = attributeSort.register[fullSortName]
               
                mainSorts.append(fullSort)
                attribSorts.append(attribSort)

        # Creates disjunctive sort for metaform if it does not already in the register of sort type names
        
        if disjunctiveSort.checkSort('rhino_shapes') == False:             # If 'rhino_shapes' corresponding sort not yet in sort register, creates sort
            fullDisjSort = disjunctiveSort('rhino_shapes', mainSorts)      # If already in sort register, retrieves sort
            
        else:
            fullDisjSort = disjunctiveSort.register['rhino_shapes']     
        return fullDisjSort, mainSorts, attribSorts   
        
    # Assigns base sort and individual name
    def mainSort(self, index, fullSortName):
        
        if index == 0:
            baseSort = pointSort3D
            fullSortName += 'point'
        elif index == 1:
            baseSort = lineSegSort3D
            if len(fullSortName) != 0: 
                fullSortName += '_lineSeg'
            else:
                fullSortName += 'lineSeg'

        return baseSort, fullSortName
        
    # Assigns attribute sort and attribute name
    def attributeSorts(self, attributes, baseSort):
        attribSort = []
        attribName = '' # Name for attribute sort
        fullAttribSort = None

        for indiv in range(len(attributes)): 
            if attributes[indiv]:
                if indiv == 0:
                    attribSort.append(labelSort)
                    attribName = attribName + 'label'
                    
                elif indiv == 1:
                    attribSort.append(descSort)
                    if len(attribName) != 0:
                        attribName = attribName + '_description'
                    else:
                        attribName = attribName + 'description'

                elif indiv == 2:
                    attribSort.append(enumSort)
                    if len(attribName) != 0:
                        attribName = attribName + '_enumerative'
                    else:
                        attribName = attribName + 'enumerative'
                
                elif indiv == 3:
                    attribSort.append(weightSort)
                    if len(attribName) != 0:
                        attribName = attribName + '_weight'
                    else:
                        attribName = attribName + 'weight'
                        
                elif indiv == 4:
                    attribSort.append(weight255Sort)
                    if len(attribName) != 0:
                        attribName = attribName + '_weight255'
                    else:
                        attribName = attribName + 'weight255'
                        
                elif indiv == 5:
                    attribSort.append(colorSort)
                    
                    if len(attribName) != 0:
                        attribName = attribName + '_color'
                    else:
                        attribName = attribName + 'color'
    
        # Creates disjunctive sort if attributes > 1
        # Else, returns attribute sort already inside attribSort
        if len(attribSort) > 1:
            if disjunctiveSort.checkSort(attribName) == False:
                fullAttribSort =  disjunctiveSort(attribName, attribSort)
            else: 
                fullAttribSort = disjunctiveSort.register[attribName]
        else:
            fullAttribSort =  attribSort[0]
        
        return attribName, attribSort, fullAttribSort

# GUIDE
# The input of tuples is as follows
# <discursiveSort instance>.parseDiscSort( <point>, <line segment>, <plane>, <volume> )

# For any given tuple of True/False values, the order of value setting is as follows
# 0. Existence of individual
# 1. Label
# 2. Description
# 3. Enumerative - values and sorting order need to be specified by user/developer; prompt is built into code
# 4. Weight - weight sort, which accepts values from 0 to 1
# 5. Weight255 - weight sort, which accepts integer values from 0 to 255
# 6. Color - color sort, which accepts RGB values (e.g. (120, 30, 40))

# EXAMPLE
# For a point with label, description, weight255 and enumerative enabled
# (True, True, True, True, True, False)
